/*******************************************************************
**	File:		superraw.c
**	Author:		David Worthen
**			Connect Tech Inc.
**
**	Original:	March 28/94
**	Revision History:
**			Dec 15/94 djw & kam fixed up 16/32 bit size problems
**			May 30/94	Revised format extensively AES
**
**	Copyright (c) 1994 CTI, Connect Tech Inc. All Rights Reserved.
**	
**	THIS IS THE UNPUBLISHED PROPRIETARY SOURCE CODE OF CONNECT TECH INC.
**	The copyright notice above does not evidence any actual or intended
**	publication of such source code.
**	
**	This module contains Proprietary Information of Connect Tech, Inc
**	and should be treated as Confidential.
**
*********************************************************************
**
**	Description:
**	This program acts as an example of how to make use of Connect Tech's
**	SUPER RAW MODE under QNX 4.xx.  This mode allows the application to
**	bypass the overhead added by QNX 4.xx's tti calls.  Seeing as this
**	function has to be called by Dev.cti for every character recieved
**	this overhead can be significant in high performance applications.
**	SUPER RAW MODE removes this overhead for applications that do not
**	need input processing (or raw data streams) by stuffing incomming 
**	characters directly into the users allocated buffer.  
**
**	NOTE: This module must be compiled with the -T 1 param.
*******************************************************************
** Version 0.00
********************************************************************/

#include <stdio.h>
#include <stddef.h>
#include <sys/kernel.h>
#include <sys/dev.h>
#include <sys/types.h>
#include <sys/proxy.h>

#if defined __386__
typedef	unsigned short *	uaddr_t;
#else
typedef	unsigned short far *	uaddr_t;
#endif

#define	CTIFNS				'U'	/* special function cti message type */
#define	SUPER_RAW_ON			8	/* super raw ON message sub type */
#define	SUPER_RAW_OFF			9	/* super raw OFF message sub type */
#define	PROXY_TRIGGER_DELTA		(0x0001)/* trigger a proxy on any new data */
#define	PROXY_TRIGGER_ZERO_DELTA	(0x0002)/* trigger a proxy on going from 0 data */

#define	MY_SIZE	10000				/* size of buffer to be used */
char	my_buffer[MY_SIZE];			/* actual buffer location */

volatile unsigned short my_counter;	/* counter of number of chars in */
					/* my_buffer */

struct	_msg	{			/* message used to pass data */
	msg_t			type;
	msg_t			sub_type;
	unsigned short 		port;
#if defined __386__
	unsigned short		ptr_offset;
	unsigned short		ptr_segment;
#else
	caddr_t			ptr;
#endif
	unsigned short 		size;
#if defined __386__
	unsigned short		counter_offset;
	unsigned short		counter_segment;
#else
	uaddr_t			counter;
#endif
	mpid_t			proxy;
	unsigned short 		mode;
} msg;

int cti_super_raw_mode( int fd, caddr_t buffer, unsigned short size,\
	uaddr_t counter, pid_t proxy, unsigned short mode ){
	
	static struct _dev_info_entry 	info;

	if ( dev_info( fd, &info ))
		return( -1 );
	
	/* create message to turn on Super Raw Mode */	
	msg.type = CTIFNS;
	msg.sub_type = SUPER_RAW_ON;
	msg.port = info.unit - 1;
#if defined __386__
	msg.ptr_offset = FP_OFF( buffer );
	msg.ptr_segment = FP_SEG( buffer );
	msg.counter_offset = FP_OFF( counter );
	msg.counter_segment = FP_SEG( counter );
#else
	msg.ptr = buffer;
	msg.counter = counter;
#endif
	msg.size = size;

	msg.proxy = proxy;
	msg.mode = mode;
    
	/* give access to Dev.cti */
	if (qnx_segment_arm( info.driver_pid, FP_SEG( buffer ), 0))
		return( -1 );

	if (qnx_segment_arm( info.driver_pid, FP_SEG( counter ), 0))
		return( -1 );

	/* send message turning on feature */
	if( Send( info.driver_pid, &msg, &msg, sizeof( struct _msg ),\
	 sizeof( struct _msg ))){
		fprintf( stderr, "Send failed\n" );
		return( -1 );
	}

	return( 0 );
}


main( int argc, char *argv[] ){
	int		temp = 0;
	pid_t		prox, pid;
	unsigned 	cc;
	int		i, index = 0;

	my_counter = 0;
	/* printf's are ment to help devloper test code */
	printf( "%s %d\n", my_buffer, my_counter );
	
	/* get a pid for the proxy */
	prox = qnx_proxy_attach( 0, 0, 0, -1 );
	
	/* turn on SUPER RAW MODE */
	if( cti_super_raw_mode( fileno(stdin), (caddr_t) my_buffer,\
	  MY_SIZE-1, (uaddr_t)&my_counter,\
	  prox, PROXY_TRIGGER_DELTA ) ){
	  	fprintf( stderr, "Error performing super raw read!\n" );
	  	exit( -1 );
	}
	
	printf( "Buffer address = %x:%x\n", FP_SEG(my_buffer),FP_OFF(my_buffer));
	printf( "%s %d\n", my_buffer, my_counter );
	
	while ( 1 ) {
		/* wait for a chunk of raw data */
		pid = Receive( 0, &cc, sizeof(unsigned));
		
		/* if my_counter is reporting data reset counter */
		_disable();
		if (my_counter) {
			temp = my_counter;
			my_counter = 0;
		}
		_enable();
		
		/* if we received something print it out */
		if (temp) {
			printf( "RX %d %05d:", temp, index );
			for(i = 0; i < temp; ++i ) {
				printf( "%02x ", my_buffer[index++] );
				if (index == (MY_SIZE-1)) index=0;
				if (((i+1)& 0xf) == 0) printf( "\n" );
			}
			printf( "\n" );
			temp = 0;
		}
	}
}

